import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class dmmf extends PApplet {

/**
Copyright 2017-2018 Yann Rollin
Contact yann.rollin@univ-nantes.fr

  This file is part of DMMF (Discrete Moment Map Flow program).

    DMMF is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DMMF is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DMMF.  If not, see <http://www.gnu.org/licenses/>.
**/    

/**
* Discrete Moment Map Flow program
* This program provides a numerical simulation of 
* an evolution equation for surfaces in the four
* dimensional space.
*
* For more details on the mathematical aspects, see our
* Research paper : Discrete Geometry and Isotropic Surfaces
* by F. Jauberteau, Y. Rollin and S. Tapie.
*/


//Global variables
PShape graph, vertices, group; //for opengl
float xmag, ymag = 0; // Mouse motions
float newXmag, newYmag = 0; //Mouse motions
int framerate = 60; 
//Initialization of the main class
Torus t=new Torus(25);

//This flag is used to trigger intro sequence
Boolean intro_flag=true;

public void setup(){
//size(1280,640,P3D);
 
  frameRate(framerate);
//default text size and color
  textSize(16);
  fill(0); 
 
 }



public void keyReleased() {
  switch(key){
      
    case 'm':
    t.movie_stop();
    break;
  }
}

public void keyPressed() {
  switch(key){
      
    case ' ':
    t.play_switch();
    break;
  
    case 'p':
    saveFrame("torus-######.tiff");
    break;
 
   case 'm':
   t.movie_rec();
   break;
    
    case 'i':
    t.showinfo_switch();
    break;
  
    case 'r':
    t.restart();
    break;
  
    case 'f':
    t.fill_switch();
    break;

    case 'b':
    t.balls_switch();
    break;

    case 'h':
    t.help_switch();
    break;

    case 't':
    t.type_prev();
    break;
 
    case 'T':
    t.type_next();
    break;

    case 'n':
    t.less_noise();
    break;

    case 'N': t.more_noise();
    break;
    
    case 'q': t.less_quads();
    break;
 
    case 'Q': t.more_quads();
    break;
 
    case 'd': t.decrease_dt();
    break;
  
    case 'D':
    t.increase_dt();
    break;

    case 's': t.scale_down();
    break;

    case 'S':
    t.scale_up();
    break;

    case 'c': t.compute_down();
    break;

    case 'C':
    t.compute_up();
    break;
  }
}

public void draw(){  
 
  //detect mouse moves 
  newXmag = mouseX/PApplet.parseFloat(width) * TWO_PI;
  newYmag = mouseY/PApplet.parseFloat(height) * TWO_PI;
  float diff = xmag-newXmag;
  if (abs(diff) >  0.01f)  
    xmag -= diff/4.0f; 
  diff = ymag-newYmag;
  if (abs(diff) >  0.01f)  
    ymag -= diff/4.0f; 
    
  // precompose 3d picture with rotations of R4
  // angles depend on mouse motions
  t.resetMatrix();
  t.rotate(1,2,-ymag); 
  t.rotate(1,3,-xmag); 

  //compute the 3 dimensional positions of points via radial projection, then stereographic projection
  t.compute3d();

  //Draw the opengl picture
  
  t.render();
  if(intro_flag==true) {
    intro(); 
  }
  
  //Apply discrete evolution of the flow
  t.evolve();
}

public void intro() {
  fill(255,0,0);
  if(millis()<10000){
    text("Discrete Moment Map Flow", 10, height -10 );
  } else if(millis()<15000){
    text("...a program after the work of...", 10, height -10 );    
  } else if(millis()<25000){
    text("Fran\u00e7ois Jauberteau, Yann Rollin and Samuel Tapie", 10, height - 10 );    
  } else if(millis()<30000){
    text("Built with Processing ", 10, height - 10);    
  } else
  intro_flag=false;
}
/**
Copyright 2017-2018 Yann Rollin
Contact yann.rollin@univ-nantes.fr

  This file is part of DMMF (Discrete Moment Map Flow program).

    DMMF is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DMMF is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DMMF.  If not, see <http://www.gnu.org/licenses/>.
**/    

//This is the main class for 
//storing a quadrangular mesh
//apply the evolution equation
//rendre the opengl picture
//and interact with parameters.

class Torus {
 V4d[][]  vertex ; // position of the vertices of the mesh
 V4d[][]  motionvector ; // this vector for each vertex is used to move the quadrangulation
 V3d[][]  vertex3d ; // coordinates projected in R3
 double[][] phi; // a function on faces that contains the symplectic area of each face
 float[][] matrix; // 4d orthogonal matrix used to precompose with 3d projections
 float timer=0.0f;//timeline parameter
 double mu_min=0.0f;//minimum symplectic density
 double mu_max =0.0f; //maximum symplectic density

//most important parameters
int N =20; //size of the quadrangulation with N^2 quads
int N_MAX=100; // for safety N must be smaller than N_MAX

int Delta_pow = -2;
double Delta = pow(10.0f,Delta_pow); // dt time step for the flow
float noise = 0.0f; // random noise parameter for initial torus
int torus_type = 0; // Clifford = 0 ; Chekanov = 1, etc..
int general_scale = 50 ; // overall scaling factor

 //Accelaration factor number of streps of 
 //computation the evolution flow
 //before drawing a picture
int  Boost_comp= 1;
 
//Overall number of initial tori implemented here 
int number_torus_types = 8; 
 
 //some colors presets to make nice gradients as a function of 
//symplectic density
int isotropic = color(0,0,255);
int isointerm = color(255,255,0);
int notiso = color (255,0,0);
float thr_max=1.0f;
float thr_interm = .5f;
float thr_min=.001f;

//vertices
int ball_size =10;
int black=color(0,0,0);
int far = color(255,0,0);
int close = color(0,0,255);
int interm = color(0,255,0);
float thr_balls_max=4.0f;
float thr_balls_interm = 2.0f;
float thr_balls_min=.1f;



//some flags for the running status
 boolean run = true; //running state (run if true or stop if false)
 boolean showinfo = false; //toggle show information
 boolean restart =true; // if true, generate a new surface
 boolean fill =true; // if true, fill the triangles
 boolean balls = false; // if true draw colored balls at the vertices
 boolean help = false; // if true show help (and override info)
 boolean movie_flag = false ; //if true save all pictures as a continuous movie
 //constructor
 Torus(){
   matrix = new float[4][4]; //rotation matrix to apply to each point   
   resetMatrix();
   resetvertex();
 }
 //another constructor 
 Torus(int k){
  N=k;
   matrix = new float[4][4]; //rotation matrix to apply to each point   
   resetMatrix();
   resetvertex();
 }
 
//Setup the mesh and allocate memory
 public void resetvertex(){
   vertex = new V4d[N][N];
   motionvector = new V4d[N][N];
   vertex3d = new V3d[N][N];
   phi = new double[N][N];

   for(int n=0 ; n<N;n++) {
   for(int m=0; m<N;m++)  {
    vertex[n][m]= new V4d();   
    motionvector[n][m]= new V4d();   
    vertex3d[n][m]= new V3d();   
     }
   }
 }
 
 //this function set the rotation matrix to the identity
 public void resetMatrix(){
 for(int i = 0;i<4;i++){ 
   for (int j = 0; j<4;j++){
   if(i==j) 
     matrix[i][j]=1.0f;
     else
      matrix[i][j]=0.0f;
     }
   }
 }
 
//mutliply the rotation matrix on the left by a rotation in the (i,j)-plane
 public void rotate(int i, int j, float phi){
  float [][] rmatrix = new float[4][4];
  float [][] oldmatrix = new float[4][4];
   
  //initialize the temporary matrix as the indentity
  // copy matrix in oldmatrix
  for(int ii=0;ii<4;ii++) {
     for(int jj=0;jj<4;jj++){
       oldmatrix[ii][jj]= matrix[ii][jj];
       if(ii==jj)
         rmatrix[ii][jj]=1.0f;
       else
         rmatrix[ii][jj]=0.0f;
       }
    }

  //modify the temporary matrix in a rotation matrix of angle phi
  rmatrix[i][i]=cos(phi);
  rmatrix[i][j]=sin(phi);
  rmatrix[j][i]=-sin(phi);
  rmatrix[j][j]=cos(phi);

  // perform matrix multiplication
  for(int ii=0;ii<4;ii++){
    for(int jj=0;jj<4;jj++){
      matrix[ii][jj]=rmatrix[ii][0]* oldmatrix[0][jj] 
        + rmatrix[ii][1]* oldmatrix[1][jj] 
        +rmatrix[ii][2]* oldmatrix[2][jj] 
        +rmatrix[ii][3]* oldmatrix[3][jj];
       }
    }
 }
 

//Implementation of several parametrizations of tori

public void create_clifford_Torus(){ 
    for (int n=0; n<N; n++){
     for (int m=0; m<N; m++){
       vertex[n][m].s=cos(2*n*PI/N)+ (random(noise)-noise/2)/N;
       vertex[n][m].t=sin(2*n*PI/N)+ (random(noise)-noise/2)/N;
       vertex[n][m].u=cos(2*m*PI/N)+ (random(noise)-noise/2)/N;
       vertex[n][m].v=sin(2*m*PI/N)+ (random(noise)-noise/2)/N;
       }
    }
 }
 
public void create_knot_Torus(int p, int q,float r)
  { 
    for (int n=0; n<N; n++){
     for (int m=0; m<N; m++){
       vertex[n][m].s=cos(2*n*PI/N*p)+ (random(noise)-noise/2)/N;
       vertex[n][m].t=sin(2*n*PI/N*p)+ (random(noise)-noise/2)/N;
       vertex[n][m].u=cos(2*n*PI/N*q)+cos(2*m*PI/N)*r+ (random(noise)-noise/2)/N;
       vertex[n][m].v=sin(2*n*PI/N*q)+sin(2*m*PI/N)*r+ (random(noise)-noise/2)/N;
       }
    }
  }
 
 //this function creates a flat torus contained in a complex line
public void create_complex_Torus(){ //noise is the level of noise of the standard clifford torus
      for (int n=0; n<N; n++){
         for (int m=0; m<N; m++){
           vertex[n][m].s= (n*2.0f-N)/N+ (random(noise)-noise/2)/N;
           vertex[n][m].t= (m*2.0f-N)/N+ (random(noise)-noise/2)/N;
           vertex[n][m].u= (random(noise)-noise/2)/N;
           vertex[n][m].v= (random(noise)-noise/2)/N;
           }
        }
   }
 
public void create_complex_Torus_B(){   
    for (int n=0; n<N; n++){
     for (int m=0; m<N; m++){
       vertex[n][m].s= (n*2.0f-N)/N+ (random(noise)-noise/2)/N;
       vertex[n][m].t= (m*2.0f-N)/N+ (random(noise)-noise/2)/N;
       vertex[n][m].u= 1.0f+(random(noise)-noise/2)/N;
       vertex[n][m].v= (random(noise)-noise/2)/N;//random(noise);
       }
      }
   }
 
public void create_double_circle() {
  float a= 2.0f;
  float b= 2.0f;
  for (int n=0; n<N; n++){
   for (int m=0; m<N; m++){
     vertex[n][m].s= - a +a * cos(PI+2*n*PI/N)+ random(noise)/N;
     vertex[n][m].t= b *sin(PI+2*n*PI/N)+ random(noise)/N;
     vertex[n][m].u=- a +a * cos(2*m*PI/N)+ random(noise)/N;
     vertex[n][m].v=  b *sin(2*m*PI/N)+ random(noise)/N;
   }
  }
}

public void create_Checkanov_Torus() {
  float xc= 1.2f;
  float yc= 1.2f;
  for (int n=0; n<N; n++){
   for (int m=0; m<N; m++){
       vertex[n][m].s=cos(2*PI*m/N) * (cos(2*n*PI/N)+xc)
       -sin(2*PI*m/N) * (sin(2*n*PI/N)+yc)+ (random(noise) - noise/2)/N +1.5f;
       vertex[n][m].t =cos(2*PI*m/N) * (sin(2*n*PI/N)+yc)
       +sin(2*PI*m/N) * (cos(2*n*PI/N)+xc)+ (random(noise)-noise/2)/N;
       vertex[n][m].u=cos(-2*PI*m/N) * (cos(2*n*PI/N)+xc)
       -sin(-2*PI*m/N) * (sin(2*n*PI/N)+yc)+ (random(noise)-noise/2)/N;
       vertex[n][m].v=cos(-2*PI*m/N) * (sin(2*n*PI/N)+yc)
       +sin(-2*PI*m/N) * (cos(2*n*PI/N)+xc)+ (random(noise)-noise/2)/N;
     }
  }
}



// This function computes the 3-dimensional representation of the torus
public void compute3d() {
    for( int n =0;n<N;n++) {
      for (int m=0;m<N;m++) {
        vertex3d[n][m].compute(vertex[n][m], matrix);
        }
      }
  }
 
 // this funtion computes the symplectic area of each face
 // and stores the result in the member phi of the class
public void computephi(){

  boolean flag_init = false;
  double mu;

  mu_min=0.0f;
  mu_max=0.0f;

   for( int n =0;n<N;n++) {
      for (int m=0;m<N;m++) {
       V4d ac= new V4d();
       V4d bd= new V4d();
       int nn = next_n(n);
       int nm = next_m(m);

       ac.bipoint(vertex[n][m],vertex[nn][nm]);
       bd.bipoint(vertex[nn][m],vertex[n][nm]);
       //   compute the symplectic area
       phi[n][m] = -.5f * ac.sympl_inner_product(bd);
       //compute the symplectic density
       mu=phi[n][m]*N*N;

      if(flag_init) {
        if(mu<mu_min)
          mu_min=mu;
        else if (mu>mu_max)
          mu_max=mu;
        } else
      {
        flag_init=true;
        mu_min = mu;
        mu_max = mu;
      }
    }
   }
 }

//This function computes the evolution vector of the flow
public void computemotionvector(){
    int nn,nm, pn,pm;
    V4d[] D = new V4d[4];
    D[0]= new V4d();
    D[1]= new V4d();
    D[2]= new V4d();
    D[3]= new V4d();

    //We compute first the symplectic area of each face
    computephi();
    //Then we compute the vector
    for( int n =0;n<N;n++) {
      for (int m=0;m<N;m++) {
        pn=prev_n(n);
        pm=prev_m(m);
        nn=next_n(n);
        nm=next_m(m);
  
        D[0].bipoint(vertex[nn][m], vertex[n][nm]);
        D[1].bipoint(vertex[n][nm], vertex[pn][m]);
        D[2].bipoint(vertex[pn][m], vertex[n][pm]);
        D[3].bipoint(vertex[n][pm], vertex[nn][m]);

        D[0].scal_mult(phi[n][m]);
        D[1].scal_mult(phi[pn][m]);
        D[2].scal_mult(phi[pn][pm]);
        D[3].scal_mult(phi[n][pm]);

        D[0].add(D[1]);
        D[0].add(D[2]);
        D[0].add(D[3]);

        D[0].J();

        motionvector[n][m].store(D[0]);
      }
    }
 }
  
  
// Some color gradient functions for faces and vertices

public int color_gradient_balls(float f){
  if (f>thr_balls_max) 
    return(far);
  if (f<thr_balls_min)
    return(black);
  if (f <thr_balls_interm)
    return(lerpColor(close,interm,f/thr_balls_interm));
  else
    return(lerpColor(interm,far,(f-thr_balls_interm)/(thr_balls_max-thr_balls_interm)));
}

public int color_gradient(float f){
  if (f>thr_max) 
    return(notiso);
  if (f<thr_min)
    return(black);
  if (f <thr_interm)
    return(lerpColor(isotropic,isointerm,f*2));
  else
   return(lerpColor(isointerm,notiso,(f-thr_interm)*2));
}



  
//this function evolves the quadrangulation along the motion vector
  
public void evolve(){  
  //check if a restart was requested
  if(restart) {
   restart=false;
   switch(torus_type) {
    case 0: create_clifford_Torus();
    break;

    case 1: create_Checkanov_Torus();
    break;

    case 2: create_complex_Torus();
    break;

    case 3: create_complex_Torus_B();
    break;
    
    case 4: create_knot_Torus(2,3,.7f);
    break;
    
    case 5: create_knot_Torus(2,5,.6f);
    break;
    
    case 6: create_knot_Torus(3,5,.5f);
    break; 
      
    case 7: create_knot_Torus(5,7,.3f);
    break;
     }
   computephi();
   timer=0;
  }

  for(int q =0; q<Boost_comp ;q++) {
    if(run){
      computemotionvector();
      for( int n =0;n<N;n++) {
        for (int m=0;m<N;m++) {
          motionvector[n][m].scal_mult(Delta);
          vertex[n][m].add(motionvector[n][m]);
          }
        }
      timer += Delta/(N*N*N*N);
        }
    }
}
  
  
 //the functions next_n and next_m return the correct index modulo N and modulo M
public int next_n(int i){
  if (i==(N-1)) return (0);
    else return(i+1);
}

public int next_m(int i){
  if (i==(N-1)) return (0);
    else return(i+1);
}
  
  //the functions prev_n and prev_m return the correct index modulo N and modulo M
public int prev_n(int i){
  if (i==0) return (N-1);
    else return(i-1);
}
  
public int prev_m(int i){
  if (i== 0) return (N-1);
    else return(i-1);
}
  
//rendering the opengl surface
public void  render() {
 //graphic opengl rendering of the 3dimensional picture  
 graph =createShape();
 if(fill) {
 graph.beginShape(QUADS);
 graph.stroke(200);
 
 for (int n = 0; n<N ; n++) {
  for (int m = 0 ; m<N ; m++) {
    int n_next,m_next;
    if (n==(N-1)) 
      n_next = 0;
    else 
      n_next = n+1;
    if (m==(N-1)) 
      m_next = 0;
    else 
      m_next = m+1;
 
 
 graph.fill(color_gradient(N*N*abs((float)t.phi[n][m])));
  
 graph.vertex(t.vertex3d[n][m].x, t.vertex3d[n][m].y, t.vertex3d[n][m].z);
 graph.vertex(t.vertex3d[n_next][m].x, t.vertex3d[n_next][m].y, t.vertex3d[n_next][m].z);
 graph.vertex(t.vertex3d[n_next][m_next].x, t.vertex3d[n_next][m_next].y, t.vertex3d[n_next][m_next].z);
 graph.vertex(t.vertex3d[n][m_next].x, t.vertex3d[n][m_next].y, t.vertex3d[n][m_next].z);
  }
 }
 
 graph.endShape();
  
}
 else {
 graph.beginShape(QUADS);
   graph.noFill();
   graph.stroke(0);
for (int n = 0; n<N ; n++) {
  for (int m = 0 ; m<N ; m++) {
    int n_next,m_next;
    if (n==(N-1)) 
      n_next = 0;
    else 
      n_next = n+1;
    if (m==(N-1)) 
      m_next = 0;
    else 
      m_next = m+1;
 
 graph.vertex(t.vertex3d[n][m].x, t.vertex3d[n][m].y, t.vertex3d[n][m].z);
 graph.vertex(t.vertex3d[n_next][m].x, t.vertex3d[n_next][m].y, t.vertex3d[n_next][m].z);
 graph.vertex(t.vertex3d[n_next][m_next].x, t.vertex3d[n_next][m_next].y, t.vertex3d[n_next][m_next].z);
 graph.vertex(t.vertex3d[n][m_next].x, t.vertex3d[n][m_next].y, t.vertex3d[n][m_next].z);
  }
 }
 
 graph.endShape();
  
 }
   
  
 if(balls) //if balls=true draw colored balls at each vertex
   {
    vertices = createShape();
    vertices.beginShape(POINTS);
    vertices.strokeWeight(ball_size);
    for (int n = 0; n<N ; n++) {
      for (int m = 0 ; m<N ; m++) {
        //define the color of verticces accordint to their distance to the origin
        float r = (float)t.vertex[n][m].normsq();
        vertices.stroke(color_gradient_balls(r));
        vertices.vertex(t.vertex3d[n][m].x,t.vertex3d[n][m].y,t.vertex3d[n][m].z);
        }
     }
    vertices.endShape();
  }

 group = createShape(GROUP);
 if(balls)  
   group.addChild(vertices);
 group.addChild(graph);
 
 background(255);
 pushMatrix(); 
 translate(width/2, height/2, -30);  
 scale(general_scale);
 shape(group);
 popMatrix();

 if(t.help){
   fill(0);
   text("Help -- Key functions", 10, 20);
   text("Space : Play/Pause", 10, 40);
   text("i : Toggle info", 10, 60);
   text("h : Toggle help", 10, 80);
   text("r : Restart", 10, 100);
   text("b : Toggle balls", 10, 120);
   text("f : Toggle fill", 10, 140);
   text("t : Switch torus type", 10, 160);
   text("d/D : Time step -/+", 10, 180);
   text("n/N : Noise level -/+", 10, 200);
   text("q/Q : Number of quads -/+", 10, 220);
   text("s/S : Overall scale -/+", 10, 240);
   text("c/C : Change computation factor -/+", 10, 260);
   text("p : Save screenshot", 10, 280);
   text("m : Press continuously to shoot a movie", 10, 300);

  }
  else if(t.showinfo){
    fill(0);
   text("Frame rate: " + PApplet.parseInt(frameRate), 10, 20);
   text("Timer : " + timer, 10, 40);
   text("Time step : " + Delta/(N*N*N*N), 10, 60);
   text("Number of quads : " + PApplet.parseInt(N*N), 10, 80);
   text("Noise level : " + noise, 10, 100);
   switch(torus_type){
    case 0:
      text("Clifford torus", 10, 120);
      break;
    case 1:
      text("Checkanov torus", 10, 120);
      break;
    case 2:
      text("Complex torus type A", 10, 120);
      break;
   case 3:
      text("Complex torus type B", 10, 120);
      break;
   case 4:
      text("(2,3)-torus knot", 10, 120);
      break;
   case 5:
      text("(2,5)-torus knot", 10, 120);
      break;
   case 6:
      text("(3,5)-torus knot", 10, 120);
      break;
   case 7:
      text("(5,7)-torus knot", 10, 120);
      break;
     }
  text("Overall scale : " + general_scale, 10, 140);
  text("Minimum symp. density : " + mu_min, 10, 160);
  text("Maximum symp. density : " + mu_max, 10, 180);
  text("Computing factor : " +  Boost_comp, 10, 200);
    }
    
 //check if a movie has to be recorded
 if(movie_flag)
   saveFrame("movie-########.tiff");
}

//Some toggle and incremental functions for parameters
public void play_switch(){
       run = ! run;
  }

public void showinfo_switch(){
    showinfo= ! showinfo; 
  }

public void restart(){
   restart=true; 
  }
  
public void fill_switch(){
    fill = ! fill; 
  }   
 
public void balls_switch(){
   balls=!balls; 
  }

public void help_switch(){
 help=!help; 
  }

public void type_next(){
  torus_type++;
  if(torus_type==number_torus_types)
      torus_type=0;  
   restart();
}

public void type_prev(){
  torus_type--;
  if(torus_type==-1)
      torus_type=number_torus_types-1;  
  restart();
}

public void less_quads(){
 if (N >2) {
      N--;
       resetvertex();
       restart(); 
    } 
}

public void more_quads(){
 if (N < N_MAX) {
      N++;
       resetvertex();
       restart(); 
    } 
}

public void decrease_dt(){
    if (Delta_pow > -10) {
      Delta_pow--;
      Delta = pow(10.0f,Delta_pow); 
    }
  
}
public void increase_dt() {
    if (Delta_pow < 2) {
      Delta_pow++;
      Delta = pow(10.0f,Delta_pow); 
    } 
}

public void less_noise(){
     if (noise < .5f)
      noise=0.0f;
      else noise-=.5f;
    restart(); 
}

public void more_noise(){
     if (noise < 50.0f)
      noise+=.5f;
    restart(); 
}

public void scale_down(){
     if (general_scale > 5) {
      general_scale-=4;
    }
}

public void scale_up(){
     if (general_scale < 100) {
      general_scale+=4;
    }
}

public void compute_down(){
    if (Boost_comp > 30) {
      Boost_comp-=10;
    }
    else if(Boost_comp>1)
      Boost_comp--;    
}

public void compute_up(){
     if (Boost_comp >= 30) 
      Boost_comp+=10; 
      else Boost_comp++;
    }
public void movie_rec(){
    movie_flag=true;
    }
public void movie_stop(){
    movie_flag=false;
    }
    
    
}
/**
Copyright 2017-2018 Yann Rollin
Contact yann.rollin@univ-nantes.fr

  This file is part of DMMF (Discrete Moment Map Flow program).

    DMMF is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DMMF is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DMMF.  If not, see <http://www.gnu.org/licenses/>.
**/    

// The class V3d defines a vector in R^3

class V3d {


 float x ;
 float y ;
 float z ;


  V3d(){
    x=0.0f;
     y=0.0f;
     z=0.0f;
  }

/*
This function rotates the vertex vert using the rotation matrix. 
The vertex is then projected on the 3-sphere then on R^3 via stereographic projection
*/
public void compute(V4d vert, float[][] matrix) {
  double ts, tt, tu, tv,r;
  //apply the rotation matrix and store the result in (ts,tt,tu,tv)
  ts = matrix[0][0] * vert.s + matrix[0][1] * vert.t + matrix[0][2] * vert.u + matrix[0][3] * vert.v ;
  tt = matrix[1][0] * vert.s + matrix[1][1] * vert.t + matrix[1][2] * vert.u + matrix[1][3] * vert.v ;
  tu = matrix[2][0] * vert.s + matrix[2][1] * vert.t + matrix[2][2] * vert.u + matrix[2][3] * vert.v ;
  tv = matrix[3][0] * vert.s + matrix[3][1] * vert.t + matrix[3][2] * vert.u + matrix[3][3] * vert.v ;

  //Project the point onto the unit 3-sphere
  r=vert.norm();
  ts = ts / r;
  tt = tt / r ; 
  tu = tu / r ; 
  tv = tv / r ; 

  //perform a stereographic projection
  
  x= (float)(ts *2 /(1-tv));
  y= (float)(tt *2 /(1-tv));
  z= (float)(tu *2 /(1-tv));
  
}

}
/**
Copyright 2017-2018 Yann Rollin
Contact yann.rollin@univ-nantes.fr

  This file is part of DMMF (Discrete Moment Map Flow program).

    DMMF is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DMMF is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DMMF.  If not, see <http://www.gnu.org/licenses/>.
**/    

// The class V4d defines a vector in R^4 identified to C^2
// real numbers are represented with double type for more accuracy

class V4d {
// s+it and u+iv are the complex coordinates
// for the identification between C^2 and R^4

 double s ;
 double t ;
 double u ;
 double v ;

V4d(){
  s=0.0f;
   t=0.0f;
   u=0.0f;
   v=0.0f;
}

 //this function stores the vector A to the current vector
V4d(V4d A){
 s = A.s; 
 t = A.t; 
 u = A.u; 
 v = A.v; 
}

//this function returns the magnitude of the vector
 public double norm(){
   return(sqrt((float)normsq()));
}

//this function returns the square of the magnitude
public double normsq(){
  return(s*s+t*t+u*u+v*v);
}

//this function computes the vector AB from the coordinates of A and B
//and stores the resut in the vector
public void bipoint(V4d A, V4d B){
 s = B.s- A.s; 
 t = B.t- A.t;
 u = B.u- A.u;
 v = B.v- A.v;
}


//this function adds the vector A to the current vector
public void add(V4d A){
 s += A.s; 
 t += A.t; 
 u += A.u; 
 v += A.v; 

}

//this function substract the vector A to the current vector
public void sub(V4d A){
 s -= A.s; 
 t -= A.t; 
 u -= A.u; 
 v -= A.v; 

}
//this function stores the vector A to the current vector
public void store(V4d A){
 s = A.s; 
 t = A.t; 
 u = A.u; 
 v = A.v; 

}

//this function rotates the vector using the complex structure J
public void J(){
 double ss=s,tt=t,uu=u,vv=v;
 s= - tt;
 t= ss;
 u= -vv;
 v= uu;
}

// computes the Euclidean inner product with another vector
public double eucl_inner_product(V4d A){
 return(s*A.s + t* A.t + u * A.u + v* A.v); 
}

// computes the symplecti inner product with another vector
// Beware that the function modifies the vector of the class
public double sympl_inner_product(V4d A){
  J();
 return(eucl_inner_product(A)); 
}

//multiply the vector by a scalar c
public void scal_mult(double c){
 s *= c;
 t *= c;
 u *= c;
 v *= c;
 
}

}
  public void settings() {  fullScreen(P3D); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "dmmf" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
